/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2009. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.rqm.defects.web.client.AbstractOSLCManager");

dojo.require("com.ibm.team.process.web.client.projectarea");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.CommonDialog");
dojo.require("com.ibm.rqm.defects.web.ui.utils.DefectsUIUtil");
dojo.require("com.ibm.rqm.defects.web.util.OSLCServicesDocUtil");
dojo.require("com.ibm.rqm.defects.web.ui.view.common.ProgressDialog");
dojo.require("com.ibm.rqm.defects.web.client.OSLCCmProvider");
dojo.require("com.ibm.rqm.defects.web.ui.view.common.ServiceProviderDialog");

(function(){
    var INIT_UNINITIALIZED = 0;
    var INIT_IN_PROGRESS = 1;
    var INIT_DONE = 2;
    var INIT_FAILURE = 3;
    
    var PROVIDER_LINK_TYPE_CQREST = "cqrest";
    
    var DefectsUIUtil = com.ibm.rqm.defects.web.ui.utils.DefectsUIUtil;
    var getCurrentProjectArea = com.ibm.team.process.web.client.projectarea.getCurrentProjectArea;
    
    // object class for cm services like picker dialogs etc 
    dojo.declare("com.ibm.rqm.defects.web.client.CMServices", null, {
        _creationDialogs: [],
        _selectionDialogs: [],
        constructor: function(paramObj){
            this._creationDialogs = paramObj.creationDialogs;
            this._selectionDialogs = paramObj.selectionDialogs;
        },
        getCreationDialog: function(id){
            var result = null;
            dojo.forEach(this._creationDialogs, function(creationDialog){
                if (creationDialog.id == id) {
                    result = creationDialog;
                }
            });
            if (result) {
                return result;
            }
            else {
                return this.getDefaultCreationDialog();
            }
        },
        
        getDefaultCreationDialog: function(){
            var result = null;
            dojo.forEach(this._creationDialogs, function(creationDialog){
                if (creationDialog.isDefault) {
                    result = creationDialog;
                }
            });
            if (result) {
                return result;
            }
            else {
                return this._creationDialogs[0];
            }
        },
        
        getSelectionDialog: function(id, isMultipleAllowed){
            var result = null;
            dojo.forEach(this._selectionDialogs, function(selectionDialog){
                if (selectionDialog.id == id) {
                    result = selectionDialog;
                }
            });
            if (!result) {
                result = this.getDefaultSelectionDialog()
            }
            if (isMultipleAllowed && result.uri.indexOf("allowsMultiple=true") == -1) {
                if (result.uri.indexOf('?') != -1) {
                    result.uri = result.uri + '&allowsMultiple=true';
                }
                else {
                    result.uri = result.uri + '?allowsMultiple=true';
                }
            }
            return result;
        },
        
        getDefaultSelectionDialog: function(){
            var result = null;
            dojo.forEach(this._selectionDialogs, function(selectionDialog){
                if (selectionDialog.isDefault) {
                    result = selectionDialog;
                }
            });
            if (result) {
                return result;
            }
            else {
                return this._selectionDialogs[0];
            }
        }
    });
    
    dojo.declare("com.ibm.rqm.defects.web.client.AbstractOSLCManager", null, {
    
        _currentProvider: null,
        
        constructor: function(){
            this._initializationState = INIT_UNINITIALIZED;
        },
        
        /* Abstract methods which the sub class has to provide implementation */
        getAPIConstructor: function(){
        },
        
        getProjectLinkType: function(){
        },
        
        getCurrentServiceProvider: function(callback){
            if (this._initializationState == INIT_UNINITIALIZED) {
                var ProgressDialog = com.ibm.rqm.defects.web.ui.view.common.ProgressDialog;
                ProgressDialog.doRunWithProgress(DefectsUIUtil.getMessage("labelInitializingDialog"), dojo.hitch(this, function(monitor){
                    monitor.beginTask(DefectsUIUtil.getMessage("msgAboutToInitializeProvider"));
                    this._chooseCurrentProvider(dojo.hitch(this, function(currentProvider){
                        monitor.done();
                        this._currentProvider = currentProvider;
                        this._initializationState = INIT_DONE;
                        if (this._currentProvider) {
                            // get api implementation 
                            var providerConstructor = this.getAPIConstructor();
                            var api = new providerConstructor(this._currentProvider);
                            this._currentProvider.postLink = dojo.hitch(api, "postLink");
                            this._currentProvider.deleteLink = dojo.hitch(api, "deleteLink");
                            // FIXME - for back compatiable only, but shouldn't use a global constant for state indicator.
                            if (this.getProjectLinkType() == "tests") {
                                com.ibm.rqm.defects.web.client.ChangeRequestManager.INIT_INITIALIZED = true;
                            }
                            else 
                                if (this.getProjectLinkType() == "validates") {
                                }
                            callback(this._currentProvider);
                        }
                        else {
                            callback(null);
                        }
                        
                    }), monitor);
                }));
                
            }
            else {
                if (this._currentProvider) {
                    callback(this._currentProvider);
                }
                else 
                    callback(null);
            }
            
        },
        
        getServiceProviders: function(callback) {
        	var dummyMonitor = {};
        	dummyMonitor.beginTask = function(label){};
            this._initProjectLinks(callback, dummyMonitor);
        },
        
        // ---- implementation -------------------------------------------------------------------------------------------------
        
        _chooseCurrentProvider: function(runAfterChosed, monitor){
            this._initProviderUris(dojo.hitch(this, function(serviceProvider){
                runAfterChosed(serviceProvider);
            }), monitor);
        },
        
        _initProviderUris: function(runAfterServiceProvidersInitiated, monitor){
            this._initProjectLinks(dojo.hitch(this, function(projectLinks){
                var projectLink = null;
                // if more than one service provider is configured, ask user to select one
                if (projectLinks && projectLinks.length > 1) {
                    var serviceProviderDialog = new com.ibm.rqm.defects.web.ui.view.common.ServiceProviderDialog({
                        onOk: dojo.hitch(this, function(currrentProvider){
                            projectLink = currrentProvider;
                            //FIXME - remove the hack
                            if (projectLink.servicesUrl.indexOf(PROVIDER_LINK_TYPE_CQREST) != -1) {
                                projectLink.linkType = PROVIDER_LINK_TYPE_CQREST;
                            }
                            monitor.beginTask(DefectsUIUtil.getMessage("msgDiscoverURIs"));
                            jazz.client.xhrGet({
                                url: projectLink.servicesUrl,
                                handleAs: "xml"
                            }).addBoth(dojo.hitch(this, function(result){
                                if (result instanceof Error) {
                                    monitor.done();
                                    this._initializationState = INIT_FAILURE;
                                    var alertDialog = null;
                                    alertDialog = new com.ibm.asq.common.web.ui.internal.widgets.AlertDialog();
                                    alertDialog.setMessage(DefectsUIUtil.getMessage("msgFailedToConfigProvider", result.message));
                                    alertDialog.open();
                                    runAfterServiceProvidersInitiated(null);
                                }
                                else {
                                    var providerUris = com.ibm.rqm.defects.web.util.OSLCServicesDocUtil.readServiceDescription(result, this.getProjectLinkType());
                                    // for back compatiable
                                    projectLink.providerUris = providerUris;
                                    projectLink.services = new com.ibm.rqm.defects.web.client.CMServices({
                                        creationDialogs: providerUris.creationDialogs,
                                        selectionDialogs: providerUris.selectionDialogs
                                    });
                                    runAfterServiceProvidersInitiated(projectLink);
                                }
                            }));
                        }),
                        serviceProviders: projectLinks
                    })
                }
                else 
                    if (projectLinks && projectLinks.length == 1) {
                        projectLink = projectLinks[0];
                        //FIXME - remove the hack
                        if (projectLink.servicesUrl.indexOf(PROVIDER_LINK_TYPE_CQREST) != -1) {
                            projectLink.linkType = PROVIDER_LINK_TYPE_CQREST;
                        }
                        monitor.beginTask(DefectsUIUtil.getMessage("msgDiscoverURIs"));
                        jazz.client.xhrGet({
                            url: projectLink.servicesUrl,
                            handleAs: "xml"
                        }).addBoth(dojo.hitch(this, function(result){
                            if (result instanceof Error) {
                                monitor.done();
                                this._initializationState = INIT_FAILURE;
                                var alertDialog = null;
                                alertDialog = new com.ibm.asq.common.web.ui.internal.widgets.AlertDialog();
                                alertDialog.setMessage(DefectsUIUtil.getMessage("msgFailedToConfigProvider", result.message));
                                alertDialog.open();
                                runAfterServiceProvidersInitiated(null);
                            }
                            else {
                                var providerUris = com.ibm.rqm.defects.web.util.OSLCServicesDocUtil.readServiceDescription(result, this.getProjectLinkType());
                                projectLink.services = new com.ibm.rqm.defects.web.client.CMServices({
                                    creationDialogs: providerUris.creationDialogs,
                                    selectionDialogs: providerUris.selectionDialogs
                                });
                                projectLink.providerUris = providerUris;
                                runAfterServiceProvidersInitiated(projectLink);
                            }
                        }));
                    }
                    
                    else {
                        runAfterServiceProvidersInitiated(null);
                    }
            }), monitor);
        },
        
        _initProjectLinks: function(runAfterProjectLinkInitiated, monitor){
            monitor.beginTask(DefectsUIUtil.getMessage("msgDiscoverProvider"));
            var linksUrl = location.protocol + "//" + location.host + "/jazz/process/project-areas/" + getCurrentProjectArea().itemId + "/links";
            var self = this;
            jazz.client.xhrGet({
                url: linksUrl,
                handleAs: "xml"
            }).addBoth(dojo.hitch(this, function(result){
                if (result instanceof Error) {
                    monitor.done();
                    this._initializationState = INIT_FAILURE;
                    var alertDialog = null;
                    alertDialog = new com.ibm.asq.common.web.ui.internal.widgets.AlertDialog();
                    alertDialog.setMessage(DefectsUIUtil.getMessage("msgFailedToConfigProvider", result.message));
                    alertDialog.open();
                    runAfterProjectLinkInitiated(null);
                }
                else {
					var projectLinks = [];
                    var links = com.ibm.rqm.defects.web.util.OSLCServicesDocUtil.readProjectLinksDescription(result, this.getProjectLinkType());
                    dojo.forEach(links, function(link){
                        projectLinks.push(link);
                    });
                    runAfterProjectLinkInitiated(projectLinks);
                }
            }));
        },
        // temp fix for CQ bridge setting up
        _initCQBridgeConfig: function(runAfterCQBridgeConfigured, monitor){
            if (this.getProjectLinkType() == "tests") {
                var serviceInterface = "com.ibm.team.repository.service.internal.IServerConfigurationRestService";
                var method = "getServiceConfiguration";
                var params = {
                    implementationClassName: "com.ibm.rational.cm.web.proxy.service.RCMConfigService"
                };
                var self = this;
                var serviceRequest = new com.ibm.team.repository.web.transport.ServiceRequest(serviceInterface, method, params);
                com.ibm.team.repository.web.transport.TeamServerClient.invokeService(serviceRequest, new com.ibm.team.repository.web.transport.ServiceResponseHandler({
                    success: function(result){
                        var hostname, repo, db;
                        dojo.forEach(result.properties, function(property){
                            if (property.name == "cqweb.server.name") {
                                hostname = property.overrideValue || property.defaultValue;
                            }
                            if (property.name == "cqweb.schema") {
                                repo = property.overrideValue;
                            }
                            if (property.name == "cqweb.userdb") {
                                db = property.overrideValue;
                            }
                        });
                        if (hostname && db && repo) {
                            var projectLink = {
                                name: repo + "/" + db,
                                linkType: "cqrest",
                                servicesUrl: location.protocol + "//" + location.host + "/jazz/cqrest/repo/" + repo + "/db/" + db
                            };
                            runAfterCQBridgeConfigured([projectLink]);
                        }
                        else {
                            runAfterCQBridgeConfigured([]);
                        }
                    },
                    fail: function(result){
                        runAfterCQBridgeConfigured([]);
                    }
                }, "success", "fail"));
            }
            else {
                runAfterCQBridgeConfigured([]);
            }
        },
        
        __sentinel: null // terminates this class definition
    });
})();
