/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.rqm.defects.web.ui.internal.WorkItemLinksTableView");

dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableConstants");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableContentProvider");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableViewer");
dojo.require("com.ibm.asq.common.web.ui.internal.view.common.ImageCache");
dojo.require("com.ibm.rqm.defects.web.ui.view.common.RichHoverCellRenderer");
dojo.require("com.ibm.rqm.defects.web.util.RichHoverUtil");

dojo.require("com.ibm.rqm.defects.web.ui.utils.DefectsUIUtil");

(function(){
	var ImageCache     			= com.ibm.asq.common.web.ui.internal.view.common.ImageCache;
    var TableConstants 			= com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableConstants;
	var RTCAdapterUIUtil 		= com.ibm.rqm.defects.web.ui.utils.DefectsUIUtil;
	var RichHoverCellRenderer 	= com.ibm.rqm.defects.web.ui.view.common.RichHoverCellRenderer;
	var RichHoverUtil		  	= com.ibm.rqm.defects.web.util.RichHoverUtil;
	
    dojo.declare("WorkItemLinksTableView.TableProvider", com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableContentProvider, {
        getNumColumns: function(content){
            return 2;
        },
        getColumnHeading: function(content, index){
            var headings = [RTCAdapterUIUtil.getMessage("labelWorkItemId"), RTCAdapterUIUtil.getMessage("labelSummaryHeading")];
            return headings[index];
        },
        getColumnHeadingImage: function(content, atColumnIndex){
            return null;
        },
        getColumnHeadingAlignment: function(atColumnIndex){
            return this.getCellAlignment(null, atColumnIndex);
        },
        getIsColumnGroupable: function(content, atColumnIndex){
            return false;
        },
        getIsColumnSortable: function(content, atColumnIndex){
            return false;
        },
        getColumnWidth: function(content, atColumnIndex){
            switch (atColumnIndex) {
                case 0:
                    return 20;
					break;
				case 1:
					return 80;
					break;
            }
            return 0;
        },
        getContentObjects: function(content){
            return content;
        },
        getRowDataId: function(rowData){
            return "_table_row_for_external_wi" + rowData.url;
        },
        getCellDisplayType: function(rowData, atColumnIndex){
            var type = TableConstants.RENDER_TYPE_TEXT;
            switch (atColumnIndex) {
                case 0:
                    type = "RENDER_TYPE_RICHHOVERLINK";
                    break;
            }
            return type;
        },
        getCellAlignment: function(rowData, atColumnIndex){
            if (atColumnIndex == 0) 
                return TableConstants.ALIGN_CENTER;
            return TableConstants.ALIGN_LEFT;
        },
        getCellDisplayData: function(rowData, atColumnIndex){
            var val = "";
            switch (atColumnIndex) {
				case 0:
					val = rowData.id;
					break;
                case 1:
                    val = this._restoreTags(rowData.summary);
                    break;
            }
            return val;
        },
        getCellDisplayExtendedInfo: function(rowData, atColumnIndex){
            var val = {};
            if (atColumnIndex == 0) {
				val.workItemUri = RichHoverUtil.getWorkItemUri(rowData.url || rowData.uri);
				val.workItemPresentationUri = RichHoverUtil.getWorkItemPresentationUri(rowData.url || rowData.uri);
				val.label = rowData.id;
            }
			
            return val;
        },
        
        // function from com.ibm.team.workitem.web.ui.internal.view.editor.EditorUtil
		_restoreTags: function(value) {
			var rawText = value;
			rawText= rawText.replace(/\&gt;/gi,">");
			rawText= rawText.replace(/\&lt;/gi,"<");
			rawText= rawText.replace(/\&amp;/gi,"&");
			rawText= rawText.replace(/\&tilde;/gi,"~");
			rawText= rawText.replace(/\&quot;/gi,"\"");
			rawText= rawText.replace(/\&circ;/gi,"^");
			rawText= rawText.replace(/\&nbsp;/gi," ");
			rawText= rawText.replace(/<br\/>/gi,"\n");
			rawText= rawText.replace(/\&#09;/g, "\t");
			return rawText
		}
    });
    
    
    dojo.declare("com.ibm.rqm.defects.web.ui.internal.WorkItemLinksTableView", [dijit._Widget, dijit._Templated], {
        templateString: '<div dojoAttachPoint="_tableViewerNode" class="com-ibm-rqm-test-artifact-table-view" tabindex="-1">' +
        '</div>',
        
        widgetsInTemplate: true,
		editable: true,
        
        constructor: function(){
        },
        
        postCreate: function(){
            this.inherited("postCreate", arguments);
            var tableViewer = new com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableViewer({
                allowAdjustSettings: true,
                autoSaveSettings: true,
                tableId: "RQM-WORKITEMLINKS-TABLEVIEWER"
            });
			tableViewer.registerCellRenderer("RENDER_TYPE_RICHHOVERLINK", new RichHoverCellRenderer());
            this._tableViewerNode.appendChild(tableViewer.domNode);
            
            var content = this.wiLinks ? this.wiLinks : [];
            tableViewer.setContentPageSize(5);
            tableViewer.setContent(content);
            var tableProvider = new WorkItemLinksTableView.TableProvider();
            tableViewer.setContentProvider(tableProvider);
            tableViewer.setTableListener(new DefectTableView.TableListener(this));
            
            tableViewer.setTableRowsCheckable(true, /* isSingleSelect */ false);
            tableViewer.show();
            for (var i = 0; i < (this.actions ? this.actions.length : 0); i++) {
                var action = this.actions[i];
                var actionButton = tableViewer.addAction("", action.label, null, action.iconClass, action.actionHandler);
				actionButton.setDisabled(!this.editable);
            }
            tableViewer.setEditable(this.editable);
            this.tableViewer = tableViewer;
        },
        
        refresh: function(links){
            this.tableViewer.setContent(links);
            this.tableViewer.refresh();
        },
		
		getCheckedRows: function() {
			return this.tableViewer.getCheckedRows();
		}
    });
    
})();
