/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.rqm.planning.web.ui.internal.module.TestPlanPickerWidget");//$NON-NLS-1$

// declare dependencies on other Dojo modules
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableContentProvider");//$NON-NLS-1$
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableConstants");//$NON-NLS-1$
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableViewer");//$NON-NLS-1$
dojo.require("jazz.ui.RemoteChooserResponder");

(function(){

    var ServiceRequest = com.ibm.team.repository.web.transport.ServiceRequest;
    var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
    var TeamServerClient = com.ibm.team.repository.web.transport.TeamServerClient;
    
    var PROCESS_SERVICE_URI = "com.ibm.team.process.internal.common.service.IProcessRestService";//$NON-NLS-1$
    var TEST_PLAN_SERVICE_URI = "com.ibm.rqm.planning.common.service.rest.ITestPlanRestService";//$NON-NLS-1$
    var TEST_PLAN_SERVICE_METHOD = "getTestPlansDTO";//$NON-NLS-1$
    var TEST_PLAN_ITEM_URL = "/jazz/secure/service/com.ibm.rqm.integration.service.IIntegrationService/testplan/";//$NON-NLS-1$
    var TableConstants = com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableConstants;
    
    dojo.declare("com.ibm.rqm.planning.web.ui.internal.module.RemoteTestPlanTableProvider", com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableContentProvider, {
        // Summary:
        //
        getNumColumns: function(content){
            return 4;
        },
        
        // Summary:
        //
        getColumnHeading: function(content, index){
            var headings = ["Id", "Name", "Owner", "Modified"];
            return headings[index];
        },
        
        getColumnWidth: function(content, atColumnIndex){
            switch (atColumnIndex) {
                case 0:
                    return 5;
                case 1:
                    return 55;
                case 2:
                    return 20;
                case 3:
                    return 20;
            }
            return 0;
        },
        
        getContentObjects: function(content){
            // our content is already an array of objects
            return content;
        },
        
        getRowDataId: function(rowData){
            return "_table_row_for_item_" + rowData.id;//$NON-NLS-1$
        },
        
        getCellDisplayType: function(rowData, atColumnIndex){
            var type = TableConstants.RENDER_TYPE_TEXT;
            
            switch (atColumnIndex) {
                case 0:
                    type = TableConstants.RENDER_TYPE_IMAGE;
                    break;
                case 4:
                    type = TableConstants.RENDER_TYPE_DATE;
                    break;
            }
            return type;
        },
        
        getCellDisplayData: function(rowData, atColumnIndex){
            var val = "data";
            switch (atColumnIndex) {
                case 0:
                    val = rowData.id; //rowData.col1;
                    break;
                case 1:
                    val = rowData.name; //rowData.col2;
                    break;
                case 2:
                    val = rowData.owner.name; //rowData.col4;
                    break;
                case 3:
                    val = new Date();
                    val.setTime(rowData.modified);
                    break;
            }
            return val;
        }
    });
    
    
    // Table view presented in the picker
    dojo.declare("com.ibm.rqm.planning.web.ui.internal.module.RemoteTestPlanTableView", [dijit._Widget, dijit._Templated], {
        templateString: '<div class="com-ibm-rqm-planning-web-module-remoteTestPlanTableView"><div dojoAttachPoint="_artifactTable"></div><div style="float:right;margin-top:0.5em;position:relative;"><button class="Button" dojoattachevent="onclick:_onOk">OK</button><button class="Button" dojoattachevent="onclick:_onCancel">Cancel</button></div></div>',
        postCreate: function(){
            this._tableViewer = new com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableViewer();
            this._artifactTable.appendChild(this._tableViewer.domNode);
            this._tableViewer.setContentProvider(new com.ibm.rqm.planning.web.ui.internal.module.RemoteTestPlanTableProvider({
                itemType: this.itemType
            }));
            this._tableViewer.setTableRowsCheckable(true, true);
            this._tableViewer.setContentPageSize(20);
            this._tableViewer.setContent(this.testplans);
            this._tableViewer.show();
        },
        _onOk: function(){
            if (this._tableViewer.getCheckedRows().length > 0) {
                this.onOk(this._tableViewer.getCheckedRows());
            }
        },
        _onCancel: function(){
            this.onCancel();
        }
    });
    
    // declare and implement the widget "class"
    dojo.declare("com.ibm.rqm.planning.web.ui.internal.module.TestPlanPickerWidget", [dijit._Widget, dijit._Templated], {
    
        currentProjectArea: null,
        
        // -- Override these to provide picker for another item in QM if you think its similar with test plan --
        getServiceUri: function(){
            return TEST_PLAN_SERVICE_URI;
        },
        
        getServiceMethod: function(){
            return TEST_PLAN_SERVICE_METHOD;
        },
        
        getItemUrl: function(id){
            return location.protocol + '//' + location.host + TEST_PLAN_ITEM_URL + "urn:com.ibm.rqm:testplan:" + id;
        },
        
        // -- Implementation --		
        templateString: '<div dojoAttachPoint="_itemPickerNode"><div class="loading">Loading...</div></div>',
        
        constructor: function(params){
            var searchString = location.search.replace(/^\?/, ''); // remove leading '?'
            this._dialogArguments = dojo.queryToObject(searchString);
        },
        
        postCreate: function(){
            // override the styles from the UI module (minWith big, blue background)
            var body = dojo.body();
            dojo.style(body, "minWidth", "200px");
            dojo.style(body, "background", "none");
            dojo.style(body, "overflow", "hidden");
            
            this.createDialog();
        },
        
        createDialog: function(){
            var srh = new ServiceResponseHandler(this, "_findProjectArea", "_noProjectArea");
            var serviceRequest = new ServiceRequest(PROCESS_SERVICE_URI, "getAllProjectAreas", {
                hideArchivedProjects: true
            });
            TeamServerClient.invokeService(serviceRequest, srh);
        },
        
        _findProjectArea: function(projectAreas){
            var name = this._dialogArguments.projectAreaName;
            
            for (var i = 0; i < projectAreas.length; i++) {
                if (name == projectAreas[i].name) {
                    this.currentProjectArea = projectAreas[i];
                }
            }
            if (this.currentProjectArea) {
                var srh = new ServiceResponseHandler(this, "_openDialog", "_openDialog");
                var serviceRequest = new ServiceRequest(this.getServiceUri(), this.getServiceMethod(), {
                    processArea: this.currentProjectArea.itemId
                });
                TeamServerClient.invokeService(serviceRequest, srh);
            }
            else {
                this._noProjectArea();
            }
        },
        
        _noProjectArea: function(){
            alert("No project area '" + this._dialogArguments.projectAreaName + "' found.");
            this._close(null);
        },
        
        _openDialog: function(testplans){
            var contentParams = {
                onOk: dojo.hitch(this, this._onOk),
                onCancel: dojo.hitch(this, this._onCancel),
                testplans: testplans
            };
            var content = new com.ibm.rqm.planning.web.ui.internal.module.RemoteTestPlanTableView(contentParams);
            
            this._setContentWidget(content);
        },
        
        _setContentWidget: function(content){
            this._itemPickerNode.removeChild(this._itemPickerNode.childNodes[0]);
            this._itemPickerNode.appendChild(content.domNode);
        },
        
        _close: function(resultArray){
            var newItems = [];
            
            if (resultArray) { // expected [ { id, name } ]
                for (var i = 0; i < resultArray.length; i++) {
                    var item = resultArray[i];
                    
                    var name = "[" + location.host + "] " + item.id + ": " + item.name;
                    var itemUrl = this.getItemUrl(item.id);
                    var newItem = {
                        "rdf:resource": itemUrl,
                        "oslc_qm:label": name
                    };
                    
                    newItems.push(newItem);
                }
            }
            
            var response = {
                "oslc_qm:message": "select",
                "oslc_qm:results": newItems
            };
			console.debug(dojo.toJson(response));
            var remoteFrame = new jazz.ui.RemoteChooserResponder();
            remoteFrame.sendResponse(dojo.toJson(response));
        },
        
        _onOk: function(testplans){
            this._close(testplans);
        },
        
        _onCancel: function(){
            this._close(null);
        }
        
    });
    
})();
