/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
console.debug("Loading: com.ibm.rqm.risk.web.ui.internal.view.assessment.AssessmentOverview");
dojo.provide("com.ibm.rqm.risk.web.ui.internal.view.assessment.AssessmentOverview");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

// Require the common widgets needed
dojo.require("com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper");
dojo.require("com.ibm.asq.common.web.ui.internal.view.common.ImageCache");

// Require local widgets
dojo.require("com.ibm.rqm.planning.web.client.internal.Constants");
dojo.require("com.ibm.rqm.planning.web.ui.internal.util.EditorUtil");
dojo.require("com.ibm.rqm.risk.web.ui.internal.view.assessment.RiskAssessmentStar");
dojo.require("com.ibm.rqm.risk.web.ui.internal.view.assessment.OverviewCommunityRiskPane");
dojo.require("com.ibm.rqm.risk.web.ui.internal.view.assessment.RiskUtil");
dojo.require("com.ibm.rqm.risk.web.client.internal.risk.RiskClient");

(function(){ // module container
    /* Jazz objects */
    var ServiceResponseHandler = com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper;
    /* QM objects */
    var Constants = com.ibm.rqm.planning.web.client.internal.Constants;
	var EditorUtil = com.ibm.rqm.planning.web.ui.internal.util.EditorUtil;
    var ImageCache = com.ibm.asq.common.web.ui.internal.view.common.ImageCache;
	var RiskAssessmentStar = com.ibm.rqm.risk.web.ui.internal.view.assessment.RiskAssessmentStar;
	var OverviewCommunityRiskPane = com.ibm.rqm.risk.web.ui.internal.view.assessment.OverviewCommunityRiskPane;
	var RiskUtil = com.ibm.rqm.risk.web.ui.internal.view.assessment.RiskUtil;
	var RiskClient = com.ibm.rqm.risk.web.client.internal.risk.RiskClient;
	var DEFAULT_COMMENT_MSG = Constants.getMessage("msgCommentHere");
	
    dojo.declare("com.ibm.rqm.risk.web.ui.internal.view.assessment.AssessmentOverview", [dijit._Widget, dijit._Templated], {
        templatePath: dojo.moduleUrl("com.ibm.rqm.risk.web", "ui/internal/view/templates/assessment/AssessmentOverview.html"),
        
		widgetsInTemplate: true,
		
        labelCommunity: Constants.getMessage("labelCommunityRiskTitle"),
		labelMine: Constants.getMessage("labelMyRiskTitle"),
		labelCalcu: Constants.getMessage("labelCalcuRiskTitle"),
		labelRateThis: Constants.getMessage("labelRateThis"),
        labelAddComment: Constants.getMessage("msgCommentHere"),
		
        // object initializer
        constructor: function(args){
			this.artifact = args.artifact;
			this.artifactType = args.artifactType;
			this.editable = args.editable ? args.editable : true;
			this.oldMyRanking = "";
			this.oldMyComment = "";
        },
        
		postCreate: function() {
			if (!this.artifact) {
                throw new Error("Parameter 'artifact' is REQUIRED!");
            }
			if(!this.artifactType) {
				throw new Error("Parameter 'artifactType' is REQUIRED!");
			}
	        this._initialize();			
		},	
		
		refresh: function() {
			this._initialize();
		},
		
        _initialize: function(){
			//set title image
			this._commImg.src = ImageCache.RISK_COMMUNITY;
			this._myImg.src = ImageCache.RISK_MYRANKING;
			this._calcuImg.src = ImageCache.RISK_CALCULATED;
			
			var args = {
				artifactType: this.artifactType,
				artifactItemId: (this.artifactType == Constants.MODEL_TYPE_REQUIREMENT)?this.artifact.itemId : this.artifact.versionableItemId,
				skipCreate: true
			}
			var responseHandler = new ServiceResponseHandler(this, "_initCommunityRisk", "_error");
			RiskClient.getCommunityRiskAssessmentDTO(responseHandler, args);
			var responseHandler = new ServiceResponseHandler(this, "_initCalcuRisk", "_error");
			RiskClient.getSystemRiskAssessmentDTO(responseHandler, args);
        },
		
		_initCommunityRisk: function(risk) {
			this.communityRisk = risk;
			this.oldMyRanking = risk.myRanking;
			var communityRisk = {
				veryHighAmount: parseInt(risk.veryHighAmount, 10),
				highAmount: parseInt(risk.highAmount, 10),
				neutralAmount: parseInt(risk.neutralAmount, 10),
				lowAmount: parseInt(risk.lowAmount, 10),
				veryLowAmount: parseInt(risk.veryLowAmount, 10),
				myRanking: parseInt(risk.myRanking, 10),
				averageLevel: parseInt(risk.riskLevel, 10)
			}
			var communityRiskWidget = new com.ibm.rqm.risk.web.ui.internal.view.assessment.OverviewCommunityRiskPane({
				risks: communityRisk
			});
			dojox.data.dom.removeChildren(this._cummunity);
			this._cummunity.appendChild(communityRiskWidget.domNode);
			
			this._initMyRisk(communityRisk);
		},
		
		_initMyRisk: function(individualRiskAssessment) {
			this._myRiskStar = this._createStarWidget(individualRiskAssessment.myRanking, this.editable, dojo.hitch(this, "_resetMyRiskLevelText"));
			dojox.data.dom.removeChildren(this._myRisk);
			this._myRisk.appendChild(this._myRiskStar.domNode);
			dojox.data.dom.removeChildren(this._myRank);
			var label = RiskUtil.getRiskLabel(individualRiskAssessment.myRanking);
			this._myRank.appendChild(document.createTextNode(label));
			RiskUtil.resetLevelLabelTextColor(this._myRank, label);
			//set my rank comment
			this._setMyRankComment();
		},
        
		_initCalcuRisk: function(systemRiskAssessment) {
			systemRiskAssessment.riskLevel = parseInt(systemRiskAssessment.riskLevel, 10);
			this._calcuRiskStar = this._createStarWidget(systemRiskAssessment.riskLevel, false);
			dojox.data.dom.removeChildren(this._calcuRisk);
			this._calcuRisk.appendChild(this._calcuRiskStar.domNode);
			dojox.data.dom.removeChildren(this._calcuRank);
			this._calcuRank.appendChild(document.createTextNode(RiskUtil.getRiskLabel(systemRiskAssessment.riskLevel)));
			RiskUtil.resetLevelLabelTextColor(this._calcuRank, RiskUtil.getRiskLabel(systemRiskAssessment.riskLevel));
		},
		
		_resetMyRiskLevelText: function(newLevel) {
			dojox.data.dom.removeChildren(this._myRank);
			var label = RiskUtil.getRiskLabel(newLevel);
			this._myRank.appendChild(document.createTextNode(label));
			RiskUtil.resetLevelLabelTextColor(this._myRank, label);
		},
		
		_setMyRankComment: function() {
            var outerDiv = document.createElement("div");
            outerDiv.id = "comment-outer-div";
			dojo.addClass(outerDiv, "comment-outer-div");
            var input = document.createElement("input");
			input.title = this.labelAddComment;
            input.name = "comment-input";
            dojo.addClass(input, "comment-input-disabled-text");
            input.type = "text";
			input.size = "20";
			
			this.currentComment = this._getMyCurrentValue(this.communityRisk.comments || []);
            input.value = this.currentComment;
            
            this._commentText = input;
            input.onfocus = dojo.hitch(this, function(){
                var value = this._commentText.value;
                this._commentText.value = value && value == this.currentComment ? "" : value;
            });
            input.onblur = dojo.hitch(this, function(){
                var value = this._commentText.value;
                this._commentText.value = value && dojo.trim(value) != "" ? value : this.currentComment;
            });
            outerDiv.appendChild(input);
			dojox.data.dom.removeChildren(this._myRankComment);
			this._myRankComment.appendChild(outerDiv);
		},
		
		_getMyCurrentValue: function(comments) {
			var currentComment = DEFAULT_COMMENT_MSG;
			var currentUserItemId = EditorUtil.getCurrentContributor().itemId;
			for(var i = comments.length - 1; i >= 0; i--) {
				if(comments[i].creator.itemId == currentUserItemId) {
					currentComment = comments[i].content;
					return currentComment;
				}
			} 
			return currentComment;
		},
		
		_createStarWidget: function(value, editable, onChangeHandler) {
			var star = new RiskAssessmentStar({
				value: value,
				editable: editable,
				onChangeHandler: onChangeHandler
			});
			return star;
		},
		
		//save my ranking when hover close
		onSave: function(table) {
			if (this._myRiskStar && this._commentText) {
				var myRanking = this._myRiskStar.getValue();
				var myComment = this._commentText.value;
				
				var args = {
					artifactType: this.artifactType,
					artifactItemId: (this.artifactType == Constants.MODEL_TYPE_REQUIREMENT)?this.artifact.itemId : this.artifact.versionableItemId
				}
				if(myComment != this.currentComment)
					args.comment = myComment;
				if(myRanking != this.oldMyRanking ){
					args.ranking = myRanking;;
				}
				if(args.comment == null && args.ranking == null)
					return;
					
				var self = this;
				var handler = {
					success: function(communityRisk){
						//refresh tableviewer's community risk
						if(table) {
							if(table.showFilters || table.showQueryFilters)
								table.runQuery(table.filterArgs);
							else if(table.refreshCallback)
								table.refreshCallback();
						}
						self.destroy();
					},
					error: function(e){
						self._error(e);
						self.destroy();
					}
				}
				var responseHandler = new ServiceResponseHandler(handler, "success", "error");
				RiskClient.postMyRanking(responseHandler, args)
			}
		},
		
		destroy: function() {
			this.inherited(arguments);
		},
		
        _error: function(e){
            console.debug("An error occurred: [" + e.message + "] @ AssessmentOverview#_error");
        }
    });
    
})();
