/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.rqm.defects.web.util.ItemPickerUtil");

(function(){
    var ITEM_KEY_MESSAGE = "oslc_cm:message";
    
    var ITEM_KEY_RESULT = "oslc_cm:results";
    var ITEM_KEY_URI = "rdf:resource";
    var ITEM_KEY_LINKTITLE = "oslc_cm:label";
	
	var OSCL_ATTRIB_TYPE= "dc:type";
	var OSCL_ATTRIB_SUMMARY= "dc:name";
	var OSCL_ATTRIB_DESCRIPION= "dc:description";
    
    /**
     * @return an array of work item objects in {id, summary, url}
     * @param {Object} workItemsJsonString - law results returned from the picker
     */
    com.ibm.rqm.defects.web.util.ItemPickerUtil.parseFromRTC = function(itemsJsonString){
        var results = [];
        var jsonObj = dojo.fromJson(itemsJsonString);
		console.info("ItemPickerUtil::parse - " + jsonObj);
        for (var index = 0; index < jsonObj[ITEM_KEY_RESULT].length; index++) {
            var itemObj = {};
            try {
            	itemObj.id = /\d+/.exec(/\] \d+/.exec(jsonObj[ITEM_KEY_RESULT][index][ITEM_KEY_LINKTITLE]))[0];
				var title = jsonObj[ITEM_KEY_RESULT][index][ITEM_KEY_LINKTITLE];
            	itemObj.summary = unescape(title.substring(title.indexOf("]") + 1, title.length ));
            } catch (e) {
            	itemObj.id = "--";
            	itemObj.summary = unescape(jsonObj[ITEM_KEY_RESULT][index][ITEM_KEY_LINKTITLE]);
            }
            itemObj.url = jsonObj[ITEM_KEY_RESULT][index][ITEM_KEY_URI];
			console.debug(itemObj);
			results.push(itemObj);
        }
        return results;
    }
	
	/**
     * @return an array of work item objects in {id, summary, url}
     * @param {Object} workItemsJsonString - law results returned from the picker
     */
    com.ibm.rqm.defects.web.util.ItemPickerUtil.parseFromCQ = function(itemsJsonString){
        var results = [];
		itemsJsonString = itemsJsonString.replace("oslc-response:", "");
        var jsonObj = dojo.fromJson(itemsJsonString);
		console.info("ItemPickerUtil::parse - " + jsonObj);
        for (var index = 0; index < jsonObj[ITEM_KEY_RESULT].length; index++) {
            var itemObj = {};
            try {
            	itemObj.id = jsonObj[ITEM_KEY_RESULT][index][ITEM_KEY_LINKTITLE].split(":")[0];
            	itemObj.summary = unescape(jsonObj[ITEM_KEY_RESULT][index][ITEM_KEY_LINKTITLE].split(":")[1]);
			} catch (e) {
				itemObj.id = "--";
				itemObj.summary = unescape(jsonObj[ITEM_KEY_RESULT][index][ITEM_KEY_LINKTITLE]);
			}
			itemObj.url = jsonObj[ITEM_KEY_RESULT][index][ITEM_KEY_URI];
			console.debug(itemObj);
			results.push(itemObj);
        }
        return results;
    }
	
	/**
	 * @return an string which would be attached to the Picker url hash fragment, the summary and description will be add to the new careated work item automaticly
	 * @param {Object} summary
	 * @param {Object} desc
	 */
	com.ibm.rqm.defects.web.util.ItemPickerUtil.getWICreationDialogParamString = function(summary, desc) {
		var params = {};
		params[OSCL_ATTRIB_SUMMARY] = summary;
		params[OSCL_ATTRIB_DESCRIPION] = desc;
		return dojo.objectToQuery(params);
	}
})();
