/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2009. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.rqm.defects.web.util.OSLCServicesDocUtil");
dojo.require("dojox.data.dom");

(function(){
    // Service discovery namespaces
    var NAMESPACE_RDF = "http://www.w3.org/1999/02/22-rdf-syntax-ns#"; //$NON-NLS-1$
    var NAMESPACE_DC = "http://purl.org/dc/terms/"; //$NON-NLS-1$
    var NAMESPACE_OSLC = "http://open-services.net/xmlns/discovery/1.0/"; //$NON-NLS-1$
    var NAMESPACE_OSLC_CM = "http://open-services.net/xmlns/cm/1.0/"; //$NON-NLS-1$
    var NAMESPACE_OSLC_QM = "http://open-services.net/xmlns/qm/1.0/"; //$NON-NLS-1$
    var NAMESPACE_OSLC_RM = "http://open-services.net/xmlns/rm/1.0/"; //$NON-NLS-1$
    var NAMESPACE_JP = "http://jazz.net/xmlns/prod/jazz/process/1.0/"; //$NON-NLS-1$
    var NAMESPACE_JD = "http://jazz.net/xmlns/prod/jazz/discovery/1.0/"; //$NON-NLS-1$
    var NAMESPACE_CALM = "http://jazz.net/xmlns/prod/jazz/calm/1.0/"; //$NON-NLS-1$
    function getURI(rootUrl, serviceDescNode){
        var uriNode = _getNode(serviceDescNode.childNodes, "url", NAMESPACE_OSLC_CM);
        if (!uriNode) {
            uriNode = _getNode(serviceDescNode.childNodes, "url", NAMESPACE_CALM)
        }
        var relativeURI = dojox.data.dom.textContent(uriNode);
        return normalizeURI(rootUrl, relativeURI);
    }
    
    function normalizeURI(rootUrl, uri){
        // Strip leading "./" if this is a relative path.
        var re = /^.\//;
        if (re.test(uri)) {
            return rootUrl + "/" + uri.replace(re, "");
        }
        re = /^(..\/)+/;
        if (re.test(uri)) {
            return rootUrl + "/" + uri.replace(re, "");
        }
        // The URI is absolute.
        return uri;
    }
    
    function readServiceDescription(serviceDescDoc, projectLinkType){
        if (projectLinkType == "tests") {
            return readCmServiceDescription(serviceDescDoc);
        }
        else 
            if (projectLinkType == "validates") {
                return readRmServiceDescription(serviceDescDoc);
            }
    }
    
    function readRmServiceDescription(serviceDescDoc, projectLinkType){
        var xmlContent = serviceDescDoc;
        var selectionDialogURI;
        var selectionPreferedWidth, selectionPreferedHeight;
        
        // get the rdf:about in case the uri is relative, it can be used as root
        var rootNode = _getNode(xmlContent.childNodes, "ServiceDescriptor", NAMESPACE_OSLC_RM);
        var requirementsNode = _getNode(rootNode.childNodes, "requirements", NAMESPACE_OSLC_RM);
        
        var selectionDialogNodes = _getNodes(requirementsNode.childNodes, "selectionDialog", NAMESPACE_OSLC_RM);
        dojo.forEach(selectionDialogNodes, function(selectionDialog){
            var widgetNode = _getNode(selectionDialog.childNodes, "widget", NAMESPACE_OSLC_RM);
            var widgetResourceAttr = _getAttributeNode(widgetNode, "Resource", NAMESPACE_RDF);
            if (widgetResourceAttr) {
                selectionDialogURI = widgetResourceAttr.value;
            }
            var widthNode = _getNode(selectionDialog.childNodes, "widthHint", NAMESPACE_OSLC_RM);
            if (widthNode) {
                selectionPreferedWidth = dojox.data.dom.textContent(widthNode);
            }
            var heightNode = _getNode(selectionDialog.childNodes, "heightHint", NAMESPACE_OSLC_RM);
            if (heightNode) {
                selectionPreferedHeight = dojox.data.dom.textContent(heightAttr);
            }
        });
        
        return {
            selectionDialog: {
                uri: selectionDialogURI,
                width: selectionPreferedWidth || '670px',
                height: selectionPreferedHeight || '600px'
            }
        };
    }
    
    /***
     * @return {Object} {defaultCreationDialogURI, selectionDialogURI}
     * @param {XMLDoc} serviceDescDoc
     */
    function readCmServiceDescription(serviceDescDoc){
        var xmlContent = serviceDescDoc;
        var defaultCreationDialogURI, defaultSelectionDialogURI, creationDialogURI, selectionDialogURI, openItemsUri, blockedItemsUri;
        var creationPreferedWidth, creationPreferedHeight, selectionPreferedWidth, selectionPreferedHeight;
        var creationDialogs = [], selectionDialogs = [];
        
        // get the rdf:about in case the uri is relative, it can be used as root
        var rootNode = _getNode(xmlContent.childNodes, "ServiceDescriptor", NAMESPACE_OSLC_CM);
        var rootUrlNode = _getAttributeNode(rootNode, "about", NAMESPACE_RDF);
        var rootUrl = _getAttributeNodeValue(rootUrlNode) ? _getAttributeNodeValue(rootUrlNode) : "./";
        var changeRequestsNode = _getNode(rootNode.childNodes, "changeRequests", NAMESPACE_OSLC_CM);
        
        var creationDialogNodes = _getNodes(changeRequestsNode.childNodes, "creationDialog", NAMESPACE_OSLC_CM);
        dojo.forEach(creationDialogNodes, function(creationDialog){
            var isDefault = false;
            var id;
            var defaultAttr = _getAttributeNode(creationDialog, "default", NAMESPACE_OSLC_CM);
            if (defaultAttr && _getAttributeNodeValue(defaultAttr) == "true") {
                isDefault = true;
                defaultCreationDialogURI = getURI(rootUrl, creationDialog);
            }
            var idAttr = _getAttributeNode(creationDialog, "id", NAMESPACE_CALM);
            if (idAttr) {
                id = _getAttributeNodeValue(idAttr);
            }
            creationDialogURI = getURI(rootUrl, creationDialog);
            var widthAttr = _getAttributeNode(creationDialog, "hintWidth", NAMESPACE_OSLC_CM);
            if (widthAttr) {
                creationPreferedWidth = _getAttributeNodeValue(widthAttr);
            }
            var heightAttr = _getAttributeNode(creationDialog, "hintHeight", NAMESPACE_OSLC_CM);
            if (heightAttr) {
                creationPreferedHeight = _getAttributeNodeValue(heightAttr);
            }
            creationDialogs.push({
                isDefault: isDefault,
                id: id,
                uri: creationDialogURI,
                width: creationPreferedWidth,
                height: creationPreferedHeight
            })
        });
        
        var selectionDialogNodes = _getNodes(changeRequestsNode.childNodes, "selectionDialog", NAMESPACE_OSLC_CM);
        dojo.forEach(selectionDialogNodes, function(selectionDialog){
            var isDefault = false;
            var id;
            var defaultAttr = _getAttributeNode(selectionDialog, "default", NAMESPACE_OSLC_CM);
            if (defaultAttr && (_getAttributeNodeValue(defaultAttr) == "true")) { // work for both IE & FF
                isDefault = true;
                defaultSelectionDialogURI = getURI(rootUrl, selectionDialog);
            }
            var idAttr = _getAttributeNode(selectionDialog, "id", NAMESPACE_CALM);
            if (idAttr) {
                id = _getAttributeNodeValue(idAttr);
            }
            selectionDialogURI = getURI(rootUrl, selectionDialog);
            var widthAttr = _getAttributeNode(selectionDialog, "hintWidth", NAMESPACE_OSLC_CM);
            if (widthAttr) {
                selectionPreferedWidth = _getAttributeNodeValue(widthAttr);
            }
            var heightAttr = _getAttributeNode(selectionDialog, "hintHeight", NAMESPACE_OSLC_CM);
            if (heightAttr) {
                selectionPreferedHeight = _getAttributeNodeValue(heightAttr);
            }
            selectionDialogs.push({
                isDefault: isDefault,
                id: id,
                uri: selectionDialogURI,
                width: selectionPreferedWidth,
                height: selectionPreferedHeight
            });
        });
        
        var filterNodes = _getNodes(changeRequestsNode.childNodes, "filter", NAMESPACE_CALM);
        dojo.forEach(filterNodes, function(filterNode){
            var idAttr = _getAttributeNode(filterNode, "id", NAMESPACE_CALM);
            if (idAttr && idAttr.value == "openItems") {
                openItemsUri = getURI(rootUrl, filterNode);
            }
            else 
                if (idAttr.value == "blockedItems") {
                    blockedItemsUri = getURI(rootUrl, filterNode);
                }
        });
        // for back compatiable
        var defaultCreationDialog, defaultSelectionDialog;
        dojo.forEach(creationDialogs, function(creationDialog){
            if (creationDialog.isDefault || creationDialog.id == "defect") {
                defaultCreationDialog = creationDialog;
            }
        });
        dojo.forEach(selectionDialogs, function(selectionDialog){
            if (selectionDialog.isDefault || selectionDialog.id == "defect") {
                defaultSelectionDialog = selectionDialog;
            }
        });
        if (!defaultCreationDialogURI) {
            defaultCreationDialog = creationDialogs[0];
            defaultCreationDialogURI = creationDialogs[0].uri;
        }
        if (!defaultSelectionDialogURI) {
            defaultSelectionDialog = selectionDialogs[0];
            defaultSelectionDialogURI = selectionDialogs[0].uri;
        }
        
        var obj = {
            defaultCreationDialogURI: defaultCreationDialogURI,
            creationDialog: defaultCreationDialog,
            creationDialogs: creationDialogs,
            selectionDialogURI: defaultSelectionDialogURI,
            selectionDialog: defaultSelectionDialog,
            selectionDialogs: selectionDialogs,
            openItemsUri: openItemsUri,
            blockedItemsUri: blockedItemsUri
        };
        return obj;
    }
    
    /***
     * @return {Object} links
     * @param {Object} projectLinkDescDoc
     */
    function readProjectLinksDescription(projectLinkDescDoc, projectLinkType){
        var links = [];
        var linksRootNode = projectLinkDescDoc.getElementsByTagName("links")[0];
        var linkNodes = _getNodes(linksRootNode.childNodes, "link", NAMESPACE_JP);
        dojo.forEach(linkNodes, function(linkNode){
            var url = null;
            var linkType = null;
            var servicesUrl = null;
            var name = null;
            dojo.forEach(linkNode.childNodes, function(childNode){
                // children of "link"
                if (childNode.tagName == "url") {
                    url = childNode.firstChild.data;
                }
                else 
                    if (childNode.tagName == "jp:link-type") {
                        linkType = childNode.firstChild.data;
                    }
                    else 
                        if (childNode.tagName == "oslc:ServiceProvider") {
                            // children of "ServiceProvider"
                            dojo.forEach(childNode.childNodes, function(childNode){
                                if (childNode.tagName == "dc:title") {
                                    name = childNode.firstChild.data;
                                }
                                else 
                                    if (childNode.tagName == "oslc:services") {
                                        servicesUrl = childNode.getAttribute("rdf:resource");
                                    }
                            });
                            
                        }
            });
            if (name && linkType == projectLinkType) {
                links.push({
                    url: url,
                    linkType: linkType,
                    servicesUrl: servicesUrl,
                    name: name
                });
            }
        });
        return links;
        
    }
    
    function _getAttributeNodeValue(attributeNode){
        return attributeNode.nodeValue;
    }
    
    function _getAttributeNode(node, baseName, namespace){
        var attribute;
        if (dojo.isIE) {
            dojo.forEach(node.attributes, function(attr){
                if (attr.nodeName.indexOf(baseName) != -1) {
                    attribute = attr;
                }
            })
        }
        else {
            attribute = node.attributes.getNamedItemNS(namespace, baseName) || node.attributes.getNamedItem(baseName);
        }
        return attribute;
    }
    
    function _getNode(collection, baseName, namespaceURI){
        var matchingNode;
        dojo.forEach(collection, function(node){
            if (dojo.isIE) {
                if (node.nodeName && node.nodeName.indexOf(baseName) != -1) {
                    matchingNode = node;
                }
            }
            else 
                if ((node.baseName == baseName || node.localName == baseName) && node.namespaceURI == namespaceURI) {
                    matchingNode = node;
                }
        });
        return matchingNode;
    }
    
    function _getNodes(collection, baseName, namespaceURI){
        var matchingNodes = [];
        dojo.forEach(collection, function(node){
            if (dojo.isIE) {
                if (node.nodeName && node.nodeName.indexOf(baseName) != -1) {
                    matchingNodes.push(node);
                }
            }
            else 
                if ((node.baseName == baseName || node.localName == baseName) && node.namespaceURI == namespaceURI) {
                    matchingNodes.push(node);
                }
        });
        return matchingNodes;
    }
    
    com.ibm.rqm.defects.web.util.OSLCServicesDocUtil.readServiceDescription = readServiceDescription;
    com.ibm.rqm.defects.web.util.OSLCServicesDocUtil.readProjectLinksDescription = readProjectLinksDescription;
})();
